/*
	         Java RayTracer
	     By Dean Camera, 2008
	http://www.fourwalledcubicle.com
*/

public class LightOmni implements ILightingObject
{
	private ColorVector LightColor;
	private Vector3D Location;
	private int DropOffDistance;

	public LightOmni(ColorVector LightColor, Vector3D Location, int DropOffDistance)
	{
		this.Location = Location;
		this.LightColor = LightColor;
		this.DropOffDistance = (DropOffDistance * DropOffDistance);
	}

	public Vector3D GetLocation()
	{
		return Location;
	}

	public ColorVector GetLightingDetails(CollisionDetails CollisionRay, float Distance)
	{
		Vector3D PointAtIncidence    = CollisionRay.GetCollisionPoint();
		Vector3D NormalToIncident    = CollisionRay.GetCollisionNormal();
		Vector3D LightToObject       = Vector3D.Subtract(PointAtIncidence, Location);
		Vector3D ObjectToViewer      = Vector3D.Subtract(PointAtIncidence, Constants.EYE_LOCATION);
		Vector3D ReflectRayDirection = LightPhysics.GetReflectionDirection(NormalToIncident, LightToObject);

		IMaterial ObjectMaterial = CollisionRay.GetCollisionObject().GetMaterial();
		
		float DiffuseComponent  = ObjectMaterial.GetDiffuse()  * Vector3D.DotProduct(NormalToIncident, LightToObject.UnitVector());
		float SpecularComponent = ObjectMaterial.GetSpecular() * (float)Math.pow(Vector3D.DotProduct(ObjectToViewer.UnitVector(), ReflectRayDirection), Constants.SPECULAR_ADJUST_POWER);

		ColorVector FinalColor = new ColorVector(0, 0, 0);
		
		if (DiffuseComponent > 0)
			FinalColor.Add(ColorVector.Multiply(LightColor, (DiffuseComponent * Constants.DIFFUSE_ADJUST)));

		if (SpecularComponent > 0)
			FinalColor.Add(ColorVector.Multiply(LightColor, (SpecularComponent * Constants.SPECULAR_ADJUST)));
		
		if (Distance > 0)
			FinalColor.Divide(Distance - DropOffDistance);
		
		return FinalColor;
	}
}