/*
	         Java RayTracer
	     By Dean Camera, 2008
	http://www.fourwalledcubicle.com
*/

public abstract class LightPhysics
{
	public static Vector3D GetReflectionDirection(Vector3D Normal, Vector3D Incident)
	{
		// Calculate reflection ray direction
		return Vector3D.Multiply(Normal, -2 * Vector3D.DotProduct(Normal, Incident)).Add(Incident).UnitVector();	
	}
	
	public static Vector3D GetRefractionDirection(Vector3D Normal, Vector3D Incident, float IndexOfRefraction)
	{
		float RefractRatio = (Constants.SCENE_REFRACT_INDEX / IndexOfRefraction);
		
		// Calculate constants used for refraction equation
		float C1 = Vector3D.DotProduct(Normal, Incident);
		float C2 = (RefractRatio * RefractRatio) * (1.0f - (C1 * C1));	

		// Check for total internal reflection - if so, no refraction color
		if (C2 > 1.0f)
		  return null;

		// Calculate refraction ray direction
		return Vector3D.Multiply(Incident, RefractRatio).Subtract(Vector3D.Multiply(Normal, RefractRatio + (float)Math.sqrt(1.0f - C2))).UnitVector();
	}
}