/*
	         Java RayTracer
	     By Dean Camera, 2008
	http://www.fourwalledcubicle.com
*/

import java.util.Random;
import java.io.IOException;

public abstract class SCENE_Balls
{
	private static Random Generator = new Random();

	public static Scene CreateScene()
	{
		// Create an array of Ray Traceable objects
		ITraceableObject[] SceneObjects = new ITraceableObject[200];

		ITexture[] ImageTex = new ITexture[10];

		System.out.println("SCENE BUILDER - Loading Textures...");

		// Load in the image textures
		for (int i = 0; i < ImageTex.length; i++)
		{
			System.out.println("SCENE BUILDER - Loading Texture " + (i + 1) + " of " + ImageTex.length);

			try
			{
				ImageTex[i] = new TexImage("Tex" + i + ".jpg", (Generator.nextInt(5) + 1), 1);
			}
			catch (IOException E)
			{
				System.out.println("Error processing texture file!");
			}
		}

		System.out.println("SCENE BUILDER - Generating Scene Objects...");

		// Texture the scene objects with random image and mathematical textures
		for (int i = 0; i < SceneObjects.length; i++)
		{
			ITexture BallTexture = null;

			int TextureIndex = Generator.nextInt(ImageTex.length) + 4;

			switch (TextureIndex)
			{
				case 10:
					BallTexture = new TexSolid(StdColors.GetNextColor());
					break;
				case 11:
					BallTexture = new TexChecker(StdColors.GetNextColor(), StdColors.GetNextColor(), Generator.nextInt(8) + 10);
					break;
				case 12:
					BallTexture = new TexStripes(StdColors.GetNextColor(), StdColors.GetNextColor(), Generator.nextInt(8) + 10, true);
					break;
				case 13:
					BallTexture = new TexStripes(StdColors.GetNextColor(), StdColors.GetNextColor(), Generator.nextInt(8) + 10, false);
					break;
				default:
					BallTexture = ImageTex[TextureIndex];
					break;
			}

			// New test sphere object
			SceneObjects[i] = new PrimSphere(Generator.nextInt(45), new Vector3D(Generator.nextInt(Constants.SCREEN_WIDTH) + 1,
																				 Generator.nextInt(Constants.SCREEN_HEIGHT) + 1,
																				 Generator.nextInt(400)), BallTexture, new MatRandom());
		}
		
		System.out.println("SCENE BUILDER - Generating Scene Lights...");

		// Create an array of Lighting objects
		ILightingObject[] SceneLights = new ILightingObject[25];

		for (int j = 0; j < SceneLights.length; j++)
		{
			ColorVector LightColor = ColorVector.Multiply(StdColors.GetNextColor(), (Generator.nextFloat() / 4) + .1f);
			Vector3D    LightPosition = new Vector3D(Generator.nextInt(Constants.SCREEN_WIDTH) + 1,
					                                 Generator.nextInt(Constants.SCREEN_HEIGHT) + 1,
					                                 Generator.nextInt(100));
		
			// New test light object
			switch(Generator.nextInt(3))
			{
				case 0:
					SceneLights[j] = new LightOmni(LightColor, LightPosition, (300 + Generator.nextInt(100)));
					break;
				case 1:
					SceneLights[j] = new LightSpotlight(LightColor, LightPosition,
					                                    new Vector3D(Generator.nextInt(Constants.SCREEN_WIDTH) + 1,
					                                                 Generator.nextInt(Constants.SCREEN_HEIGHT) + 1,
					                                                 Generator.nextInt(400)),
					                                    (float)(Math.PI / (7 + Generator.nextInt(3))));
					break;
				case 2:
					SceneLights[j] = new LightFloodlight(LightColor, LightPosition,
					                                     new Vector3D(Generator.nextInt(Constants.SCREEN_WIDTH) + 1,
					                                                  Generator.nextInt(Constants.SCREEN_HEIGHT) + 1,
					                                                  Generator.nextInt(400)),
					                                     (float)(Math.PI / (7 + Generator.nextInt(3))));
					break;				
			}
		}

		System.out.println("SCENE BUILDER - Done.");

		return new Scene("Test Ball Scene", SceneObjects, SceneLights, new ColorVector(Generator.nextInt(255), Generator.nextInt(255), Generator.nextInt(255)),
		                 new ColorVector(255, 255, 255).Multiply(.3f));
	}
}

