/*
	         Java RayTracer
	     By Dean Camera, 2008
	http://www.fourwalledcubicle.com
*/

public final class Vector3D
{
	public static final Vector3D UNITVECTORX = new Vector3D(1, 0, 0);
	public static final Vector3D UNITVECTORY = new Vector3D(0, 1, 0);
	public static final Vector3D UNITVECTORZ = new Vector3D(0, 0, 1);

	private float X, Y, Z;

	public Vector3D(float X, float Y, float Z)
	{
		this.X = X;
		this.Y = Y;
		this.Z = Z;
	}

	public Vector3D(Vector3D V)
	{
		this.X = V.GetX();
		this.Y = V.GetY();
		this.Z = V.GetZ();
	}
	
	public float GetX()
	{
		return X;
	}

	public float GetY()
	{
		return Y;
	}

	public float GetZ()
	{
		return Z;
	}

	public void SetX(float NewX)
	{
		X = NewX;
	}

	public void SetY(float NewY)
	{
		Y = NewY;
	}

	public void SetZ(float NewZ)
	{
		Z = NewZ;
	}

	public void Set(Vector3D V)
	{
		X = V.GetX();
		Y = V.GetY();
		Z = V.GetZ();
	}

	public void Set(float NewX, float NewY, float NewZ)
	{
		X = NewX;
		Y = NewY;
		Z = NewZ;
	}

	public float GetLength()
	{
		return (float)Math.sqrt((X * X) + (Y * Y) + (Z * Z));
	}

	public float GetLengthSquared()
	{
		return (X * X) + (Y * Y) + (Z * Z);
	}

	public Vector3D UnitVector()
	{
		float Length = GetLength();

		if (Length != 0)
			Divide(GetLength());
		else
			Set(0, 0, 0);

		return this;
	}

	public Vector3D Add(Vector3D V)
	{
		X += V.GetX();
		Y += V.GetY();
		Z += V.GetZ();

		return this;
	}

	public Vector3D Add(float AddX, float AddY, float AddZ)
	{
		X += AddX;
		Y += AddY;
		Z += AddZ;

		return this;
	}

	public Vector3D Add(float Scalar)
	{
		X += Scalar;
		Y += Scalar;
		Z += Scalar;

		return this;
	}

	public Vector3D Subtract(Vector3D V)
	{
		X -= V.GetX();
		Y -= V.GetY();
		Z -= V.GetZ();

		return this;
	}

	public Vector3D Subtract(float SubX, float SubY, float SubZ)
	{
		X -= SubX;
		Y -= SubY;
		Z -= SubZ;

		return this;
	}

	public Vector3D Subtract(float Scalar)
	{
		X -= Scalar;
		Y -= Scalar;
		Z -= Scalar;

		return this;
	}

	public Vector3D Multiply(float MulX, float MulY, float MulZ)
	{
		X *= MulX;
		Y *= MulY;
		Z *= MulZ;

		return this;
	}
	
	public Vector3D Multiply(float Scaler)
	{
		X *= Scaler;
		Y *= Scaler;
		Z *= Scaler;

		return this;
	}

	public Vector3D Divide(float DivX, float DivY, float DivZ)
	{
		X *= DivX;
		Y *= DivY;
		Z *= DivZ;

		return this;
	}
	
	public Vector3D Divide(float Scaler)
	{
		X /= Scaler;
		Y /= Scaler;
		Z /= Scaler;

		return this;
	}

	public static Vector3D UnitVector(Vector3D V)
	{
		return new Vector3D(V).UnitVector();
	}

	public static float DotProduct(Vector3D V1, Vector3D V2)
	{
		return ((V1.GetX() * V2.GetX()) + (V1.GetY() * V2.GetY()) + (V1.GetZ() * V2.GetZ()));
	}
	
	public static Vector3D CrossProduct(Vector3D V1, Vector3D V2)
	{
		return new Vector3D((V1.GetY() * V2.GetZ()) - (V1.GetZ() * V2.GetY()),
		                    (V1.GetZ() * V2.GetX()) - (V1.GetX() * V2.GetZ()),
		                    (V1.GetX() * V2.GetY()) - (V1.GetY() * V2.GetX()));
	}

	public static Vector3D Multiply(Vector3D V, float Scalar)
	{
		return new Vector3D(V).Multiply(Scalar);
	}

	public static Vector3D Multiply(Vector3D V, float MulX, float MulY, float MulZ)
	{
		return new Vector3D(V).Multiply(MulX, MulY, MulZ);
	}
	
	public static Vector3D Divide(Vector3D V, float Scalar)
	{
		return new Vector3D(V).Divide(Scalar);
	}

	public static Vector3D Divide(Vector3D V, float DivX, float DivY, float DivZ)
	{
		return new Vector3D(V).Divide(DivX, DivY, DivZ);
	}

	public static Vector3D Add(Vector3D V, float Scalar)
	{
		return new Vector3D(V).Add(Scalar);
	}

	public static Vector3D Add(Vector3D V, float AddX, float AddY, float AddZ)
	{
		return new Vector3D(V).Add(AddX, AddY, AddZ);
	}

	public static Vector3D Add(Vector3D V1, Vector3D V2)
	{
		return new Vector3D(V1).Add(V2);
	}

	public static Vector3D Subtract(Vector3D V, float Scalar)
	{
		return new Vector3D(V).Subtract(Scalar);
	}

	public static Vector3D Subtract(Vector3D V, float SubX, float SubY, float SubZ)
	{
		return new Vector3D(V).Subtract(SubX, SubY, SubZ);
	}

	public static Vector3D Subtract(Vector3D V1, Vector3D V2)
	{
		return new Vector3D(V1).Subtract(V2);
	}
}
